import java.net.*;   // Import the names of the classes
import java.io.*;    // to be used.

/**
 * This application creates a single, concise HTML page
 * of hits from the AltaVista search engine given a 
 * search string.
 * @author David W. Baker
 * @version 1.1
 */
 class AltaVistaList {
   private static final String AGENT_NAME = 
      "java-alta-search";
   private static final String AGENT_VERSION = "1.0";
   private static final String SEARCH_URL = 
       "http://WWW.OMEN.RU/";
   private int totalHits = 0;
   private StringBuffer outputList = new StringBuffer();

   /**
    * This starts the application.
    * @param args Program arguments - the search string.
    */
   public static void main(String[] args) {
      if (args.length == 0) {
         System.out.println(
            "Usage: AltaVistaList search string");
         System.exit(1);
      }
      AltaVistaList runApp = new AltaVistaList(args);
      runApp.printOutput(System.out);
      System.exit(0);
   }

   /**
    * This constructor connects to AltaVista and obtains
    * all of the relevant hits.
    * @param args The search tokens.
    */
   public AltaVistaList(String[] args) {
      String hitData;      // Store incoming data.
      int startHits = 0;   // Get the next 10 hits from here.
      String searchSyntax = createQuery(args);
    
      URLConnection.setDefaultRequestProperty("User-Agent",
         AGENT_NAME + "/" + AGENT_VERSION);
      while (true) {
         hitData = getPage(SEARCH_URL + "?" + searchSyntax + 
            startHits); // Go get a page of hits.
         hitData = getHits(hitData);  // Extract the hits.
         // If there were no hits in the page, hitData will
         // be null. If there were hits, append them to
         // the outputList, increment to the next 10 hits,
         // and go through the loop again.
         if (hitData != null) {
            outputList.append(hitData + "\n");
            startHits += 10;
         // Otherwise, break from the loop.
         } else {
            break;
         }
      }
   }

   /**
    * This method builds an AltaVista search query 
    * string.
    * @param searchTokens An array of search tokens.
    * @return The search query string built.
    */ 
   protected String createQuery(String[] searchTokens) {
      StringBuffer searchString = new StringBuffer();
    
      // Apend the tokens to a single string.
      for(int index = 0; index < searchTokens.length; 
            index++) {
         searchString.append(searchTokens[index]);
         // Add a space if there's another token coming up.
         if (index < searchTokens.length-1) {
            searchString.append(" ");
         }
      }
      // URL encode the string.
      String encodedSearchString = 
         URLEncoder.encode(searchString.toString());
      // Return the proper query string.
      return "what=web&fmt=c&pg=q&q=" + encodedSearchString 
         + "&stq=";
   }

   /**
    * This method obtains a page from the Web.
    * @param url The URL of the page to obtain.
    * @return The page obtained.
    */
   protected String getPage(String url) {
      // A buffer for the incoming page.
      StringBuffer page = new StringBuffer();
      String nextLine;  // The next line in the input stream.
    
      try {
         URL urlObject = new URL(url);
         URLConnection agent = urlObject.openConnection();
         DataInputStream input = 
            new DataInputStream(agent.getInputStream());
         // While readLine() doesn't return null, append
         // then next line to the buffer.
         while((nextLine = input.readLine()) != null) {
            page.append(nextLine+"\n");
         }
         input.close();
      } catch(MalformedURLException excpt) {
         System.out.println("Badly formed URL: " + excpt);
      } catch(IOException excpt) {
         System.out.println("Failed I/O: " + excpt);
      }
      // Convert the buffer to a string and return.
      return page.toString();
   }

   /**
    * This method extracts the list of hits from a returned
    * AltaVista results page.
    * @param hitPage The page returned from AltaVista.
    * @return The list of hits.
    */
   protected String getHits(String hitPage) {
      int first,last;     // Begin/end of a substring.
      int notFound = -1;  // Not found return for indexOf().
      String hitSection = null;  // The hits part of page.

      // Go to the first "<a href=" after "<pre>".
      first = hitPage.indexOf("<pre>") + "<pre>".length();
      first = hitPage.indexOf("<a href=",first);
      // End pointer at "</pre>".
      last = hitPage.indexOf("</pre>");

      // If our beginning is after our end, return.    
      if (last < first) {
         return hitSection;
      }
      // If neither substring is found, return.
      if (first == notFound || last == notFound) {
         System.err.println("Bad search page format");
         return hitSection;
      }

      // Cut out the substring.    
      hitSection = hitPage.substring(first,last);
      first = last = 0;
      totalHits += 1;   // Found one hit.
      // Go through the page line by line.
      while((last = hitSection.indexOf("\n",first)) 
               != notFound) {
         // Find the next "<a href=" which should be 
         // immediately after the \n.
         first = hitSection.indexOf("<a href=",last);
         // If it's not, return the current substring.
         if (first != (last+1)) {
            return hitSection.substring(0,last);
         // Otherwise, another hit has been found.
         } else {
            totalHits += 1;
         }
      }
      return hitSection; // Return the substring.
   }

   /**
    * This method prints the list of hits obtained from
    * AltaVista.
    * @param sendOutput Where to print the output.
    */
   public void printOutput(PrintStream sendOutput) {
      sendOutput.print("<!DOCTYPE HTML PUBLIC \"-//IETF//" +
         "DTD HTML//EN\">\n<HTML>\n<HEAD>\n<TITLE>" + 
         AGENT_NAME + "</TITLE>\n</HEAD>\n<BODY>\n<H1>" +
         "Search Results</H1>\n<P><STRONG>Total number of " +
         "hits: " + totalHits + "</STRONG></P>\n<PRE>\n" + 
         outputList + "</PRE>\n</BODY>\n</HTML>\n");
   }
}

